#!/bin/bash
### BEGIN INIT INFO
# Provides:          dashcore-init.sh
# Required-Start:    networking
# Required-Stop:     networking
# Should-Start:      sshd tor
# Should-Stop:       sshd tor
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: dashd init script
# Description:       This file should be used to construct scripts to be
#                    placed in /etc/init.d.
### END INIT INFO

PATH=/sbin:/usr/sbin:/bin:/usr/bin
DESC="Init dashcore"
NAME=dashcore-init.sh
DAEMON=/usr/lib/dashcore/dashd
DASH_CLI=/usr/lib/dashcore/dash-cli
DAEMON_ARGS=""
SCRIPTNAME=/etc/init.d/$(/usr/bin/basename $0)
DEFAULT_CONF=/etc/default/dashcore-init

. /lib/init/vars.sh
. /lib/lsb/init-functions

# Exit if the package is not installed
[ -x "$DAEMON" ] || exit 0

if [ -f ${DEFAULT_CONF} ]
then
    . ${DEFAULT_CONF}
fi

set -e

LOG_FILE=${DATADIR}/$( /usr/bin/basename ${DAEMON} ).log
TESTNET_LOG_FILE=${TESTNET_DATADIR}/$( /usr/bin/basename ${DAEMON} ).log

if [ -z "${RUN_VERSION}" ]
then
    RUN_VERSION=main
fi

if ! [ "${ENABLE}" = "yes" ]
then
    log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
    exit 0
fi

#
# Function that starts the daemon/service
#
do_start()
{
    INIT_TYPE="$1"
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	case "${RUN_VERSION}" in
	    main)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${DATADIR} -conf=${SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
		/bin/sleep 4
	    ;;
	    testnet)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${TESTNET_DATADIR} -conf=${TESTNET_SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${TESTNET_LOG_FILE}"
		RESULT=$?
		/bin/sleep 4
	    ;;
	    both)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${DATADIR} -conf=${SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
		/bin/sleep 1
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${TESTNET_DATADIR} -conf=${TESTNET_SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${TESTNET_LOG_FILE}"
		RESULT=$[ ${RESULT} + $? ]
		/bin/sleep 4
	    ;;
	esac
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

do_start_reindex()
{
    RESULT=0
    do_start -reindex
    RESULT=$?
    return ${RESULT}
}

do_start_rescan()
{
    RESULT=0
    do_start -rescan
    RESULT=$?
    return ${RESULT}
}

#
# Function that stops the daemon/service
#
do_stop()
{
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	case "${RUN_VERSION}" in
	    main)
		PIDO=`/bin/cat ${DATADIR}/dashd.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${SERVER_CONF} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		/bin/sleep 9
	    ;;
	    testnet)
		TESTNET_PIDO=`/bin/cat ${TESTNET_DATADIR}/testnet3/dashd.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${TESTNET_SERVER_CONF} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		/bin/sleep 9
	    ;;
	    both)
		PIDO=`/bin/cat ${DATADIR}/dashd.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${SERVER_CONF} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		/bin/sleep 1
		TESTNET_PIDO=`/bin/cat ${TESTNET_DATADIR}/testnet3/dashd.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${TESTNET_SERVER_CONF} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$[ ${RESULT} + $? ]
		/bin/sleep 9
	    ;;
	esac

	COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	COUNTER=0
	while [ ${COUNT_PROC} -gt 0 ]
	do
	    COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	    COUNTER=$[ ${COUNTER} + 1 ]
	    if [ ${COUNTER} -ge 95 ]
	    then
		COUNT_PROC=0
	    fi
	    /bin/sleep 1
	done
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that sends a SIGHUP to the daemon/service
#
do_reload()
{
    do_stop
    /bin/sleep 4
    do_start
}

force_start()
{
    RESULT=0

    case "${RUN_VERSION}" in
	main)
	    /bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${DATADIR} -conf=${SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
	    RESULT=$?
	    /bin/sleep 4
	;;
	testnet)
	    /bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${TESTNET_DATADIR} -conf=${TESTNET_SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
	    RESULT=$?
	    /bin/sleep 4
	;;
	both)
	    /bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${DATADIR} -conf=${SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
	    RESULT=$?
	    /bin/sleep 1
	    /bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -datadir=${TESTNET_DATADIR} -conf=${TESTNET_SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
	    RESULT=$[ ${RESULT} + $? ]
	    /bin/sleep 4
	;;
    esac

    return ${RESULT}
}

force_stop()
{
    RESULT=0

    case "${RUN_VERSION}" in
	main)
	    PIDO=`/bin/cat ${DATADIR}/dashd.pid 2>/dev/null`
	    /bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${SERVER_CONF} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
	    RESULT=$?
	    /bin/sleep 9
	;;
	testnet)
	    TESTNET_PIDO=`/bin/cat ${TESTNET_DATADIR}/testnet3/dashd.pid 2>/dev/null`
	    /bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${TESTNET_SERVER_CONF} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
	    RESULT=$?
	    /bin/sleep 9
	;;
	both)
	    PIDO=`/bin/cat ${DATADIR}/dashd.pid 2>/dev/null`
	    /bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${SERVER_CONF} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
	    RESULT=$?
	    /bin/sleep 1
	    TESTNET_PIDO=`/bin/cat ${TESTNET_DATADIR}/testnet3/dashd.pid 2>/dev/null`
	    /bin/su ${RUNUSER} -c "${DASH_CLI} -conf=${TESTNET_SERVER_CONF} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
	    RESULT=$[ ${RESULT} + $? ]
	    /bin/sleep 9
	;;
    esac

    COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
    COUNTER=0
    while [ ${COUNT_PROC} -gt 0 ]
    do
	COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	COUNTER=$[ ${COUNTER} + 1 ]
	if [ ${COUNTER} -ge 95 ]
	then
	    COUNT_PROC=0
	fi
	/bin/sleep 1
    done

    return ${RESULT}
}

force_restart()
{
    force_stop
    /bin/sleep 4
    force_start
}

RESULT=0

case "$1" in
  start)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start dashd"
	do_start
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  start-reindex)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start dashd reindex mode"
	do_start_reindex
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  start-rescan)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start dashd rescan mode"
	do_start_rescan
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  stop)
	log_daemon_msg "${NAME}"
	log_progress_msg "Stop dashd"
	do_stop
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  force-start)
	log_daemon_msg "${NAME}"
	log_progress_msg "Force start dashd"
	force_start
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  force-stop)
	log_daemon_msg "${NAME}"
	log_progress_msg "Force stop dashd"
	force_stop
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  reload|force-reload)
	log_daemon_msg "${NAME}"
	log_progress_msg "Reload dashd"
	do_reload
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  restart|force-reload)
	log_daemon_msg "${NAME}"
	log_progress_msg "Restart dashd"
	do_reload
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  force-restart)
	log_daemon_msg "${NAME}"
	log_progress_msg "Force restart dashd"
	force_restart
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  *)
	echo "Usage: $SCRIPTNAME {start|stop|restart|force-reload|force-start|force-stop|force-restart|start-reindex|start-rescan}" >&2
	exit 3
	;;
esac

exit ${RESULT}
